@@ -1,5 +1,3 @@
-import sys
-
 from django.conf import settings
 
 
@@ -21,5 +19,3 @@ class AppSettings:
 
 
 app_settings = AppSettings()
-app_settings.__name__ = __name__
-sys.modules[__name__] = app_settings
